package cn.com.countrygarden.bi.main.dao.twerp.hm;

import cn.com.countrygarden.bi.main.model.twerp.hm.TbDesktopResource;
import org.springframework.stereotype.Repository;

import javax.transaction.Transactional;
import java.util.List;
import java.util.Map;

@Repository
@Transactional
public interface TbDesktopResourceMapper {
    List<TbDesktopResource> sumAll();
    TbDesktopResource sumByUserCode(String userCode);
    TbDesktopResource sumByUserCodeAndOrganCode(Map<String, Object> param);
    TbDesktopResource sumByUserCodeAndOrganCodeAndCommID(Map<String, Object> param);

    List<TbDesktopResource> sumByOrganByUserCode(String userCode);
    List<TbDesktopResource> sumByOrganByUserCodeAndOrganCode(Map<String, Object> param);
    List<TbDesktopResource> sumByOrganByOrganByUserCodeAndOrganCodeAndCommID(Map<String, Object> param);

    List<TbDesktopResource> sumPropertyFormatByUserCode(Map<String, String> param);

    List<TbDesktopResource> sumDevelopmentSubjectByUserCode(Map<String, String> param);

    List<TbDesktopResource> sumByUserCodeByProvince(Map<String, String> param);
}
